@@ -17,7 +17,7 @@
 // Copyright (C) 2006, 2007 Jeff Muizelaar <jeff@infidigm.net>
 // Copyright (C) 2006 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2009 Koji Otani <sho@bbr.jp>
-// Copyright (C) 2009-2011, 2013, 2016-2022, 2024 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2009-2011, 2013, 2016-2022, 2024, 2025 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2010 Christian Feuersänger <cfeuersaenger@googlemail.com>
 // Copyright (C) 2011 Andrea Canciani <ranma42@gmail.com>
 // Copyright (C) 2011-2014, 2016, 2020 Thomas Freitag <Thomas.Freitag@alfa.de>
@@ -29,6 +29,7 @@
 // Copyright (C) 2024 Athul Raj Kollareth <krathul3152@gmail.com>
 // Copyright (C) 2024 Nelson Benítez León <nbenitezl@gmail.com>
 // Copyright (C) 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2025 Trystan Mata <trystan.mata@tytanium.xyz>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -44,6 +45,7 @@
 #include "Object.h"
 #include "Function.h"
 
+#include <array>
 #include <cassert>
 #include <map>
 #include <memory>
@@ -277,7 +279,7 @@ public:
     virtual void getDeviceNLine(unsigned char * /*in*/, unsigned char * /*out*/, int /*length*/) { error(errInternal, -1, "GfxColorSpace::getDeviceNLine this should not happen"); }
 
     // create mapping for spot colorants
-    virtual void createMapping(std::vector<std::unique_ptr<GfxSeparationColorSpace>> *separationList, int maxSepComps);
+    virtual void createMapping(std::vector<std::unique_ptr<GfxSeparationColorSpace>> *separationList, size_t maxSepComps);
     const std::vector<int> &getMapping() const { return mapping; }
 
     // Does this ColorSpace support getRGBLine?
@@ -364,7 +366,7 @@ public:
     GfxColorSpaceMode getMode() const override { return csCalGray; }
 
     // Construct a CalGray color space.  Returns nullptr if unsuccessful.
-    static std::unique_ptr<GfxColorSpace> parse(Array *arr, GfxState *state);
+    static std::unique_ptr<GfxColorSpace> parse(const Array &arr, GfxState *state);
 
     void getGray(const GfxColor *color, GfxGray *gray) const override;
     void getRGB(const GfxColor *color, GfxRGB *rgb) const override;
@@ -460,7 +462,7 @@ public:
     GfxColorSpaceMode getMode() const override { return csCalRGB; }
 
     // Construct a CalRGB color space.  Returns nullptr if unsuccessful.
-    static std::unique_ptr<GfxColorSpace> parse(Array *arr, GfxState *state);
+    static std::unique_ptr<GfxColorSpace> parse(const Array &arr, GfxState *state);
 
     void getGray(const GfxColor *color, GfxGray *gray) const override;
     void getRGB(const GfxColor *color, GfxRGB *rgb) const override;
@@ -480,13 +482,13 @@ public:
     double getGammaR() const { return gammaR; }
     double getGammaG() const { return gammaG; }
     double getGammaB() const { return gammaB; }
-    const double *getMatrix() const { return mat; }
+    const std::array<double, 9> &getMatrix() const { return mat; }
 
 private:
     double whiteX, whiteY, whiteZ; // white point
     double blackX, blackY, blackZ; // black point
     double gammaR, gammaG, gammaB; // gamma values
-    double mat[9]; // ABC -> XYZ transform matrix
+    std::array<double, 9> mat; // ABC -> XYZ transform matrix
     void getXYZ(const GfxColor *color, double *pX, double *pY, double *pZ) const;
 #ifdef USE_CMS
     std::shared_ptr<GfxColorTransform> transform;
@@ -537,7 +539,7 @@ public:
     GfxColorSpaceMode getMode() const override { return csLab; }
 
     // Construct a Lab color space.  Returns nullptr if unsuccessful.
-    static std::unique_ptr<GfxColorSpace> parse(Array *arr, GfxState *state);
+    static std::unique_ptr<GfxColorSpace> parse(const Array &arr, GfxState *state);
 
     void getGray(const GfxColor *color, GfxGray *gray) const override;
     void getRGB(const GfxColor *color, GfxRGB *rgb) const override;
@@ -586,7 +588,7 @@ public:
     std::unique_ptr<GfxICCBasedColorSpace> copyAsOwnType() const;
 
     // Construct an ICCBased color space.  Returns nullptr if unsuccessful.
-    static std::unique_ptr<GfxColorSpace> parse(Array *arr, OutputDev *out, GfxState *state, int recursion);
+    static std::unique_ptr<GfxColorSpace> parse(const Array &arr, OutputDev *out, GfxState *state, int recursion);
 
     void getGray(const GfxColor *color, GfxGray *gray) const override;
     void getRGB(const GfxColor *color, GfxRGB *rgb) const override;
@@ -645,7 +647,7 @@ public:
     GfxColorSpaceMode getMode() const override { return csIndexed; }
 
     // Construct an Indexed color space.  Returns nullptr if unsuccessful.
-    static std::unique_ptr<GfxColorSpace> parse(GfxResources *res, Array *arr, OutputDev *out, GfxState *state, int recursion);
+    static std::unique_ptr<GfxColorSpace> parse(GfxResources *res, const Array &arr, OutputDev *out, GfxState *state, int recursion);
 
     void getGray(const GfxColor *color, GfxGray *gray) const override;
     void getRGB(const GfxColor *color, GfxRGB *rgb) const override;
@@ -672,7 +674,7 @@ public:
     unsigned char *getLookup() { return lookup; }
     GfxColor *mapColorToBase(const GfxColor *color, GfxColor *baseColor) const;
     unsigned int getOverprintMask() const { return base->getOverprintMask(); }
-    void createMapping(std::vector<std::unique_ptr<GfxSeparationColorSpace>> *separationList, int maxSepComps) override { base->createMapping(separationList, maxSepComps); }
+    void createMapping(std::vector<std::unique_ptr<GfxSeparationColorSpace>> *separationList, size_t maxSepComps) override { base->createMapping(separationList, maxSepComps); }
 
 private:
     std::unique_ptr<GfxColorSpace> base; // base color space
@@ -699,14 +701,14 @@ public:
     std::unique_ptr<GfxSeparationColorSpace> copyAsOwnType() const;
 
     // Construct a Separation color space.  Returns nullptr if unsuccessful.
-    static std::unique_ptr<GfxColorSpace> parse(GfxResources *res, Array *arr, OutputDev *out, GfxState *state, int recursion);
+    static std::unique_ptr<GfxColorSpace> parse(GfxResources *res, const Array &arr, OutputDev *out, GfxState *state, int recursion);
 
     void getGray(const GfxColor *color, GfxGray *gray) const override;
     void getRGB(const GfxColor *color, GfxRGB *rgb) const override;
     void getCMYK(const GfxColor *color, GfxCMYK *cmyk) const override;
     void getDeviceN(const GfxColor *color, GfxColor *deviceN) const override;
 
-    void createMapping(std::vector<std::unique_ptr<GfxSeparationColorSpace>> *separationList, int maxSepComps) override;
+    void createMapping(std::vector<std::unique_ptr<GfxSeparationColorSpace>> *separationList, size_t maxSepComps) override;
 
     int getNComps() const override { return 1; }
     void getDefaultColor(GfxColor *color) const override;
@@ -744,14 +746,14 @@ public:
     GfxColorSpaceMode getMode() const override { return csDeviceN; }
 
     // Construct a DeviceN color space.  Returns nullptr if unsuccessful.
-    static std::unique_ptr<GfxColorSpace> parse(GfxResources *res, Array *arr, OutputDev *out, GfxState *state, int recursion);
+    static std::unique_ptr<GfxColorSpace> parse(GfxResources *res, const Array &arr, OutputDev *out, GfxState *state, int recursion);
 
     void getGray(const GfxColor *color, GfxGray *gray) const override;
     void getRGB(const GfxColor *color, GfxRGB *rgb) const override;
     void getCMYK(const GfxColor *color, GfxCMYK *cmyk) const override;
     void getDeviceN(const GfxColor *color, GfxColor *deviceN) const override;
 
-    void createMapping(std::vector<std::unique_ptr<GfxSeparationColorSpace>> *separationList, int maxSepComps) override;
+    void createMapping(std::vector<std::unique_ptr<GfxSeparationColorSpace>> *separationList, size_t maxSepComps) override;
 
     int getNComps() const override { return nComps; }
     void getDefaultColor(GfxColor *color) const override;
@@ -788,7 +790,7 @@ public:
     GfxColorSpaceMode getMode() const override { return csPattern; }
 
     // Construct a Pattern color space.  Returns nullptr if unsuccessful.
-    static std::unique_ptr<GfxColorSpace> parse(GfxResources *res, Array *arr, OutputDev *out, GfxState *state, int recursion);
+    static std::unique_ptr<GfxColorSpace> parse(GfxResources *res, const Array &arr, OutputDev *out, GfxState *state, int recursion);
 
     void getGray(const GfxColor *color, GfxGray *gray) const override;
     void getRGB(const GfxColor *color, GfxRGB *rgb) const override;
@@ -845,22 +847,22 @@ public:
 
     int getPaintType() const { return paintType; }
     int getTilingType() const { return tilingType; }
-    const double *getBBox() const { return bbox; }
+    const std::array<double, 4> &getBBox() const { return bbox; }
     double getXStep() const { return xStep; }
     double getYStep() const { return yStep; }
     Dict *getResDict() { return resDict.isDict() ? resDict.getDict() : (Dict *)nullptr; }
-    const double *getMatrix() const { return matrix; }
+    const std::array<double, 6> &getMatrix() const { return matrix; }
     Object *getContentStream() { return &contentStream; }
 
 private:
-    GfxTilingPattern(int paintTypeA, int tilingTypeA, const double *bboxA, double xStepA, double yStepA, const Object *resDictA, const double *matrixA, const Object *contentStreamA, int patternRefNumA);
+    GfxTilingPattern(int paintTypeA, int tilingTypeA, const std::array<double, 4> &bboxA, double xStepA, double yStepA, const Object *resDictA, const std::array<double, 6> &matrixA, const Object *contentStreamA, int patternRefNumA);
 
     int paintType;
     int tilingType;
-    double bbox[4];
+    const std::array<double, 4> bbox;
     double xStep, yStep;
     Object resDict;
-    double matrix[6];
+    const std::array<double, 6> matrix;
     Object contentStream;
 };
 
@@ -877,13 +879,13 @@ public:
     std::unique_ptr<GfxPattern> copy() const override;
 
     GfxShading *getShading() { return shading.get(); }
-    const double *getMatrix() const { return matrix; }
+    const std::array<double, 6> &getMatrix() const { return matrix; }
 
 private:
-    GfxShadingPattern(std::unique_ptr<GfxShading> &&shadingA, const double *matrixA, int patternRefNumA);
+    GfxShadingPattern(std::unique_ptr<GfxShading> &&shadingA, const std::array<double, 6> &matrixA, int patternRefNumA);
 
     std::unique_ptr<GfxShading> shading;
-    double matrix[6];
+    const std::array<double, 6> matrix;
 };
 
 //------------------------------------------------------------------------
@@ -987,7 +989,7 @@ private:
 class POPPLER_PRIVATE_EXPORT GfxFunctionShading : public GfxShading
 {
 public:
-    GfxFunctionShading(double x0A, double y0A, double x1A, double y1A, const double *matrixA, std::vector<std::unique_ptr<Function>> &&funcsA);
+    GfxFunctionShading(double x0A, double y0A, double x1A, double y1A, const std::array<double, 6> &matrixA, std::vector<std::unique_ptr<Function>> &&funcsA);
     explicit GfxFunctionShading(const GfxFunctionShading *shading);
     ~GfxFunctionShading() override;
 
@@ -1002,7 +1004,7 @@ public:
         *x1A = x1;
         *y1A = y1;
     }
-    const double *getMatrix() const { return matrix; }
+    const std::array<double, 6> &getMatrix() const { return matrix; }
     int getNFuncs() const { return funcs.size(); }
     const Function *getFunc(int i) const { return funcs[i].get(); }
     void getColor(double x, double y, GfxColor *color) const;
@@ -1012,7 +1014,7 @@ protected:
 
 private:
     double x0, y0, x1, y1;
-    double matrix[6];
+    const std::array<double, 6> matrix;
     std::vector<std::unique_ptr<Function>> funcs;
 };
 
@@ -1421,6 +1423,38 @@ private:
     GfxPath(bool justMoved1, double firstX1, double firstY1, GfxSubpath **subpaths1, int n1, int size1);
 };
 
+//------------------------------------------------------------------------
+// GfxXYZ2DisplayTransforms
+//------------------------------------------------------------------------
+
+#ifdef USE_CMS
+
+class POPPLER_PRIVATE_EXPORT GfxXYZ2DisplayTransforms
+{
+public:
+    // Constructor.
+    explicit GfxXYZ2DisplayTransforms(const GfxLCMSProfilePtr &displayProfileA);
+
+    // Accessors.
+    GfxLCMSProfilePtr getDisplayProfile() const { return displayProfile; }
+    std::shared_ptr<GfxColorTransform> getRelCol() const { return XYZ2DisplayTransformRelCol; }
+    std::shared_ptr<GfxColorTransform> getAbsCol() const { return XYZ2DisplayTransformAbsCol; }
+    std::shared_ptr<GfxColorTransform> getSat() const { return XYZ2DisplayTransformSat; }
+    std::shared_ptr<GfxColorTransform> getPerc() const { return XYZ2DisplayTransformPerc; }
+
+private:
+    static GfxLCMSProfilePtr XYZProfile;
+
+    GfxLCMSProfilePtr displayProfile;
+
+    std::shared_ptr<GfxColorTransform> XYZ2DisplayTransformRelCol;
+    std::shared_ptr<GfxColorTransform> XYZ2DisplayTransformAbsCol;
+    std::shared_ptr<GfxColorTransform> XYZ2DisplayTransformSat;
+    std::shared_ptr<GfxColorTransform> XYZ2DisplayTransformPerc;
+};
+
+#endif
+
 //------------------------------------------------------------------------
 // GfxState
 //------------------------------------------------------------------------
@@ -1573,6 +1607,8 @@ public:
     void setPath(GfxPath *pathA);
     double getCurX() const { return curX; }
     double getCurY() const { return curY; }
+    double getCurTextX() const { return curTextX; }
+    double getCurTextY() const { return curTextY; }
     void getClipBBox(double *xMin, double *yMin, double *xMax, double *yMax) const
     {
         *xMin = clipXMin;
@@ -1661,6 +1697,7 @@ public:
 #ifdef USE_CMS
     void setDisplayProfile(const GfxLCMSProfilePtr &localDisplayProfileA);
     GfxLCMSProfilePtr getDisplayProfile() { return localDisplayProfile; }
+    void setXYZ2DisplayTransforms(std::shared_ptr<GfxXYZ2DisplayTransforms> transforms);
     std::shared_ptr<GfxColorTransform> getXYZ2DisplayTransform();
     int getCmsRenderingIntent();
     static GfxLCMSProfilePtr sRGBProfile;
@@ -1714,19 +1751,14 @@ public:
     void clipToRect(double xMin, double yMin, double xMax, double yMax);
 
     // Text position.
-    void textSetPos(double tx, double ty)
-    {
-        lineX = tx;
-        lineY = ty;
-    }
     void textMoveTo(double tx, double ty)
     {
         lineX = tx;
         lineY = ty;
-        textTransform(tx, ty, &curX, &curY);
+        textTransform(tx, ty, &curTextX, &curTextY);
     }
     void textShift(double tx, double ty);
-    void shift(double dx, double dy);
+    void textShiftWithUserCoords(double dx, double dy);
 
     // Push/pop GfxState on/off stack.
     GfxState *save();
@@ -1785,7 +1817,11 @@ private:
     int render; // text rendering mode
 
     GfxPath *path; // array of path elements
+    // Ideally we would not have curX and curTextX, but there are broken PDF producers that mix operators incorrectly
+    // and given that Adobe Reader renders them correctly we have decided to have two sets of coordinates to fix/workaround
+    // the rendering of those files
     double curX, curY; // current point (user coords)
+    double curTextX, curTextY; // start of current text line (user coords)
     double lineX, lineY; // start of current text line (text coords)
 
     double clipXMin, clipYMin, // bounding box for clip region
@@ -1798,11 +1834,7 @@ private:
 
 #ifdef USE_CMS
     GfxLCMSProfilePtr localDisplayProfile;
-    std::shared_ptr<GfxColorTransform> XYZ2DisplayTransformRelCol;
-    std::shared_ptr<GfxColorTransform> XYZ2DisplayTransformAbsCol;
-    std::shared_ptr<GfxColorTransform> XYZ2DisplayTransformSat;
-    std::shared_ptr<GfxColorTransform> XYZ2DisplayTransformPerc;
-    static GfxLCMSProfilePtr XYZProfile;
+    std::shared_ptr<GfxXYZ2DisplayTransforms> XYZ2DisplayTransforms;
 #endif
 
     std::unique_ptr<GfxColorSpace> defaultGrayColorSpace;
