@@ -17,12 +17,14 @@
 // Copyright (C) 2008 Hugo Mercier <hmercier31@gmail.com>
 // Copyright (C) 2010, 2011 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2012 Tobias Koening <tobias.koenig@kdab.com>
-// Copyright (C) 2018-2020 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2018-2023, 2025 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
 // Copyright (C) 2018 Intevation GmbH <intevation@intevation.de>
 // Copyright (C) 2019, 2020 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright (C) 2020 Adam Reichold <adam.reichold@t-online.de>
 // Copyright (C) 2020 Marek Kasik <mkasik@redhat.com>
+// Copyright (C) 2024 Pratham Gandhi <ppg.1382@gmail.com>
+// Copyright (C) 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -33,7 +35,9 @@
 #define LINK_H
 
 #include "Object.h"
+#include "poppler_private_export.h"
 #include <memory>
+#include <optional>
 #include <set>
 
 class GooString;
@@ -62,10 +66,11 @@ enum LinkActionKind
     actionOCGState, // Set-OCG-State action
     actionHide, // Hide action
     actionResetForm, // ResetForm action
+    actionSubmitForm, // SubmitForm action
     actionUnknown // anything else
 };
 
-class LinkAction
+class POPPLER_PRIVATE_EXPORT LinkAction
 {
 public:
     LinkAction();
@@ -85,13 +90,13 @@ public:
     static std::unique_ptr<LinkAction> parseDest(const Object *obj);
 
     // Parse an action dictionary.
-    static std::unique_ptr<LinkAction> parseAction(const Object *obj, const GooString *baseURI = nullptr);
+    static std::unique_ptr<LinkAction> parseAction(const Object *obj, const std::optional<std::string> &baseURI = {});
 
     // A List of the next actions to execute in order.
     const std::vector<std::unique_ptr<LinkAction>> &nextActions() const;
 
 private:
-    static std::unique_ptr<LinkAction> parseAction(const Object *obj, const GooString *baseURI, std::set<int> *seenNextActions);
+    static std::unique_ptr<LinkAction> parseAction(const Object *obj, const std::optional<std::string> &baseURI, std::set<int> *seenNextActions);
 
     std::vector<std::unique_ptr<LinkAction>> nextActionList;
 };
@@ -112,14 +117,11 @@ enum LinkDestKind
     destFitBV
 };
 
-class LinkDest
+class POPPLER_PRIVATE_EXPORT LinkDest
 {
 public:
     // Build a LinkDest from the array.
-    LinkDest(const Array *a);
-
-    // Copy a LinkDest.
-    LinkDest *copy() const { return new LinkDest(this); }
+    explicit LinkDest(const Array &a);
 
     // Was the LinkDest created successfully?
     bool isOk() const { return ok; }
@@ -153,19 +155,17 @@ private:
                      //   destFitH/BH use changeTop;
                      //   destFitV/BV use changeLeft
     bool ok; // set if created successfully
-
-    LinkDest(const LinkDest *dest);
 };
 
 //------------------------------------------------------------------------
 // LinkGoTo
 //------------------------------------------------------------------------
 
-class LinkGoTo : public LinkAction
+class POPPLER_PRIVATE_EXPORT LinkGoTo : public LinkAction
 {
 public:
     // Build a LinkGoTo from a destination (dictionary, name, or string).
-    LinkGoTo(const Object *destObj);
+    explicit LinkGoTo(const Object *destObj);
 
     ~LinkGoTo() override;
 
@@ -222,7 +222,7 @@ class LinkLaunch : public LinkAction
 {
 public:
     // Build a LinkLaunch from an action dictionary.
-    LinkLaunch(const Object *actionObj);
+    explicit LinkLaunch(const Object *actionObj);
     ~LinkLaunch() override;
 
     // Was the LinkLaunch created successfully?
@@ -242,11 +242,11 @@ private:
 // LinkURI
 //------------------------------------------------------------------------
 
-class LinkURI : public LinkAction
+class POPPLER_PRIVATE_EXPORT LinkURI : public LinkAction
 {
 public:
     // Build a LinkURI given the URI (string) and base URI.
-    LinkURI(const Object *uriObj, const GooString *baseURI);
+    LinkURI(const Object *uriObj, const std::optional<std::string> &baseURI);
 
     ~LinkURI() override;
 
@@ -270,7 +270,7 @@ class LinkNamed : public LinkAction
 {
 public:
     // Build a LinkNamed given the action name.
-    LinkNamed(const Object *nameObj);
+    explicit LinkNamed(const Object *nameObj);
 
     ~LinkNamed() override;
 
@@ -299,7 +299,7 @@ public:
         operationTypeStop
     };
 
-    LinkMovie(const Object *obj);
+    explicit LinkMovie(const Object *obj);
 
     ~LinkMovie() override;
 
@@ -343,7 +343,7 @@ public:
         ResumeRendition
     };
 
-    LinkRendition(const Object *Obj);
+    explicit LinkRendition(const Object *Obj);
 
     ~LinkRendition() override;
 
@@ -376,7 +376,7 @@ private:
 class LinkSound : public LinkAction
 {
 public:
-    LinkSound(const Object *soundObj);
+    explicit LinkSound(const Object *soundObj);
 
     ~LinkSound() override;
 
@@ -406,7 +406,7 @@ class LinkJavaScript : public LinkAction
 {
 public:
     // Build a LinkJavaScript given the action name.
-    LinkJavaScript(Object *jsObj);
+    explicit LinkJavaScript(Object *jsObj);
 
     ~LinkJavaScript() override;
 
@@ -415,7 +415,7 @@ public:
     LinkActionKind getKind() const override { return actionJavaScript; }
     const std::string &getScript() const { return js; }
 
-    static Object createObject(XRef *xref, const GooString &js);
+    static Object createObject(XRef *xref, const std::string &js);
 
 private:
     std::string js;
@@ -428,7 +428,7 @@ private:
 class LinkOCGState : public LinkAction
 {
 public:
-    LinkOCGState(const Object *obj);
+    explicit LinkOCGState(const Object *obj);
 
     ~LinkOCGState() override;
 
@@ -466,7 +466,7 @@ private:
 class LinkHide : public LinkAction
 {
 public:
-    LinkHide(const Object *hideObj);
+    explicit LinkHide(const Object *hideObj);
 
     ~LinkHide() override;
 
@@ -498,11 +498,11 @@ private:
 // LinkResetForm
 //------------------------------------------------------------------------
 
-class LinkResetForm : public LinkAction
+class POPPLER_PRIVATE_EXPORT LinkResetForm : public LinkAction
 {
 public:
     // Build a LinkResetForm.
-    LinkResetForm(const Object *nameObj);
+    explicit LinkResetForm(const Object *nameObj);
 
     ~LinkResetForm() override;
 
@@ -518,6 +518,50 @@ private:
     bool exclude;
 };
 
+//------------------------------------------------------------------------
+// LinkSubmitForm
+//------------------------------------------------------------------------
+
+class POPPLER_PRIVATE_EXPORT LinkSubmitForm : public LinkAction
+{
+public:
+    enum SubmitFormFlag
+    {
+        NoOpFlag = 0,
+        ExcludeFlag = 1,
+        IncludeNoValueFieldsFlag = 1 << 1,
+        ExportFormatFlag = 1 << 2,
+        GetMethodFlag = 1 << 3,
+        SubmitCoordinatesFlag = 1 << 4,
+        XFDFFlag = 1 << 5,
+        IncludeAppendSavesFlag = 1 << 6,
+        IncludeAnnotationsFlag = 1 << 7,
+        SubmitPDFFlag = 1 << 8,
+        CanonicalFormatFlag = 1 << 9,
+        ExclNonUserAnnotsFlag = 1 << 10,
+        ExclFKeyFlag = 1 << 11,
+        // 13th high bit flag is undefined
+        EmbedFormFlag = 1 << 13,
+    };
+
+    // Build a LinkSubmitForm
+    explicit LinkSubmitForm(const Object *submitObj);
+
+    ~LinkSubmitForm() override;
+
+    bool isOk() const override { return !url.empty(); }
+
+    LinkActionKind getKind() const override { return actionSubmitForm; }
+    const std::vector<std::string> &getFields() const { return fields; };
+    const std::string &getUrl() const { return url; };
+    uint32_t getFlags() const { return flags; };
+
+private:
+    std::vector<std::string> fields;
+    std::string url;
+    uint32_t flags = 0;
+};
+
 //------------------------------------------------------------------------
 // LinkUnknown
 //------------------------------------------------------------------------
@@ -526,7 +570,7 @@ class LinkUnknown : public LinkAction
 {
 public:
     // Build a LinkUnknown with the specified action type.
-    LinkUnknown(const char *actionA);
+    explicit LinkUnknown(std::string &&actionA);
 
     ~LinkUnknown() override;
 
@@ -546,11 +590,11 @@ private:
 // Links
 //------------------------------------------------------------------------
 
-class Links
+class POPPLER_PRIVATE_EXPORT Links
 {
 public:
     // Extract links from array of annotations.
-    Links(Annots *annots);
+    explicit Links(Annots *annots);
 
     // Destructor.
     ~Links();
@@ -558,12 +602,10 @@ public:
     Links(const Links &) = delete;
     Links &operator=(const Links &) = delete;
 
-    // Iterate through list of links.
-    int getNumLinks() const { return links.size(); }
-    AnnotLink *getLink(int i) const { return links[i]; }
+    const std::vector<std::shared_ptr<AnnotLink>> &getLinks() const { return links; }
 
 private:
-    std::vector<AnnotLink *> links;
+    std::vector<std::shared_ptr<AnnotLink>> links;
 };
 
 #endif
